@@ -1,59 +1,48 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement
 import os
-import sys
 import time
 import errno
 import hashlib
 
 from mapproxy.util.lock import FileLock, DummyLock, cleanup_lockdir
 from mapproxy.image import ImageSource, is_single_color_image
-from mapproxy.config import base_config
 
 import logging
-log = logging.getLogger(__name__)
+log = logging.getLogger('mapproxy.cache.file')
 
 class FileCache(object):
     """
     This class is responsible to store and load the actual tile data.
     """
-    def __init__(self, cache_dir, file_ext, lock_dir=None, pre_store_filter=None,
-                 link_single_color_images=False):
+    def __init__(self, cache_dir, file_ext, lock_dir=None,
+                 link_single_color_images=False, lock_timeout=60.0):
         """
         :param cache_dir: the path where the tile will be stored
         :param file_ext: the file extension that will be appended to
             each tile (e.g. 'png')
-        :param pre_store_filter: a list with filter. each filter will be called
-            with a tile before it will be stored to disc. the filter should 
-            return this or a new tile object.
         """
         self.cache_dir = cache_dir
         if lock_dir is None:
             lock_dir = os.path.join(cache_dir, 'tile_locks')
         self.lock_dir = lock_dir
+        self.lock_timeout = lock_timeout
         self.file_ext = file_ext
         self._lock_cache_id = None
-        if pre_store_filter is None:
-            pre_store_filter = []
-        self.pre_store_filter = pre_store_filter
-        if link_single_color_images and sys.platform == 'win32':
-            log.warn('link_single_color_images not supported on windows')
-            link_single_color_images = False
         self.link_single_color_images = link_single_color_images
     
     def level_location(self, level):
@@ -64,7 +53,10 @@ class FileCache(object):
         >>> c.level_location(2)
         '/tmp/cache/02'
         """
-        return os.path.join(self.cache_dir, "%02d" % level)
+        if isinstance(level, basestring):
+            return os.path.join(self.cache_dir, level)
+        else:
+            return os.path.join(self.cache_dir, "%02d" % level)
     
     def tile_location(self, tile, create_dir=False):
         """
@@ -164,9 +156,6 @@ class FileCache(object):
         """
         Add the given `tile` to the file cache. Stores the `Tile.source` to
         `FileCache.tile_location`.
-        
-        All ``pre_store_filter`` will be called with the tile, before
-        it will be stored.
         """
         if tile.stored:
             return
@@ -197,10 +186,8 @@ class FileCache(object):
     def _store(self, tile, location):
         if os.path.islink(location):
             os.unlink(location)
-        
-        for img_filter in self.pre_store_filter:
-            tile = img_filter(tile)
-        data = tile.source.as_buffer(format=self.file_ext, seekable=True)
+            
+        data = tile.source.as_buffer(seekable=True)
         data.seek(0)
         with open(location, 'wb') as f:
             log.debug('writing %r to %s' % (tile.coord, location))
@@ -225,7 +212,7 @@ class FileCache(object):
         """
         lock_filename = self.lock_filename(tile)
         cleanup_lockdir(self.lock_dir, force=False)
-        return FileLock(lock_filename, timeout=base_config().http.client_timeout,
+        return FileLock(lock_filename, timeout=self.lock_timeout,
             remove_on_unlock=True)
     
     def __repr__(self):
