@@ -416,20 +416,32 @@ def make_transparent(img, color, tolerance=10):
     :param tolerance: tolerance applied to each color value
     """
     result = _make_transparent(img.as_image(), color, tolerance)
-    return ImageSource(result, size=result.size, transparent=True)
+    image_opts = img.image_opts.copy()
+    image_opts.transparent = True
+    image_opts.mode = 'RGBA'
+    return ImageSource(result, size=result.size, image_opts=image_opts)
 
 def _make_transparent(img, color, tolerance=10):
     img.load()
     
     if img.mode == 'P':
-        img = img.convert('RGB')
+        img = img.convert('RGBA')
     
     channels = img.split()
     mask_channels = []
     for ch, c in zip(channels, color):
+        # create bit mask for each matched color
         low_c, high_c = c-tolerance, c+tolerance
         mask_channels.append(Image.eval(ch, lambda x: 255 if low_c <= x <= high_c else 0))
-        
+    
+    # multiply channel bit masks to get a single mask
     alpha = reduce(ImageChops.multiply, mask_channels)
-    img.putalpha(ImageChops.invert(alpha))
+    # invert to get alpha channel
+    alpha = ImageChops.invert(alpha)
+
+    if len(channels) == 4:
+        # multiply with existing alpha
+        alpha = ImageChops.multiply(alpha, channels[-1])
+    
+    img.putalpha(alpha)
     return img
