@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 WMS service handler
@@ -26,8 +25,8 @@ from mapproxy.service.base import Server
 from mapproxy.response import Response
 from mapproxy.source import SourceError
 from mapproxy.exception import RequestError
-from mapproxy.config import base_config
 from mapproxy.image import concat_legends, LayerMerger
+from mapproxy.image.opts import ImageOptions
 from mapproxy.image.message import attribution_image, message_image
 from mapproxy.layer import BlankImage, MapQuery, InfoQuery, LegendQuery, MapError
 from mapproxy.layer import MapBBOXError, merge_layer_extents, merge_layer_res_ranges
@@ -38,19 +37,17 @@ from mapproxy.service import template_helper
 
 get_template = template_loader(__file__, 'templates', namespace=template_helper.__dict__)
 
-import logging
-log = logging.getLogger(__name__)
 
 class PERMIT_ALL_LAYERS(object):
     pass
 
 class WMSServer(Server):
-    names = ('service',)
-    request_methods = ('map', 'capabilities', 'featureinfo', 'legendgraphic')
+    service = 'wms'
     fi_transformers = None
     
-    def __init__(self, root_layer, md, layer_merger=None, request_parser=None, tile_layers=None,
-        attribution=None, srs=None, image_formats=None, info_types=None, strict=False, on_error='raise',
+    def __init__(self, root_layer, md, srs, image_formats, layer_merger=None,
+        request_parser=None, tile_layers=None, attribution=None, 
+        info_types=None, strict=False, on_error='raise',
         concurrent_layer_renderer=1):
         Server.__init__(self)
         self.request_parser = request_parser or partial(wms_request, strict=strict)
@@ -65,12 +62,12 @@ class WMSServer(Server):
         self.md = md
         self.on_error = on_error
         self.concurrent_layer_renderer = concurrent_layer_renderer
-        self.image_formats = image_formats or base_config().wms.image_formats
+        self.image_formats = image_formats
         self.info_types = info_types
-        self.srs = srs or base_config().wms.srs
+        self.srs = srs
                 
     def map(self, map_request):
-        self.check_request(map_request)
+        self.check_map_request(map_request)
         
         params = map_request.params
         query = MapQuery(params.bbox, params.size, SRS(params.srs), params.format)
@@ -90,7 +87,8 @@ class WMSServer(Server):
                 for layer_name, map_layers in layer.map_layers_for_query(query):
                     actual_layers[layer_name] = map_layers
         
-        authorized_map_layers = self.authorized_map_layers(actual_layers.keys(), map_request.http.environ)
+        authorized_map_layers = self.authorized_map_layers(actual_layers.keys(),
+            map_request.http.environ, query_extent=(query.srs.srs_code, query.bbox))
         
         if authorized_map_layers is not PERMIT_ALL_LAYERS:
             requested_layer_names = set(map_request.params.layers)
@@ -112,13 +110,13 @@ class WMSServer(Server):
                                  concurrent_rendering=self.concurrent_layer_renderer)
         renderer.render(merger)
         
-        if self.attribution:
+        if self.attribution and not query.tiled_only:
             merger.add(attribution_image(self.attribution['text'], params.size))
-        result = merger.merge(params.format, params.size,
-                              bgcolor=params.bgcolor,
-                              transparent=params.transparent)
-        return Response(result.as_buffer(format=params.format),
-                        content_type=params.format_mime_type)
+        img_opts = self.image_formats[params.format_mime_type].copy()
+        img_opts.bgcolor = params.bgcolor
+        img_opts.transparent = params.transparent
+        result = merger.merge(size=params.size, image_opts=img_opts)
+        return Response(result.as_buffer(img_opts), content_type=img_opts.format.mime_type)
     
     def capabilities(self, map_request):
         # TODO: debug layer
@@ -148,7 +146,7 @@ class WMSServer(Server):
     
     def featureinfo(self, request):
         infos = []
-        self.check_request(request)
+        self.check_featureinfo_request(request)
         
         p = request.params
         query = InfoQuery(p.bbox, p.size, SRS(p.srs), p.pos,
@@ -158,14 +156,13 @@ class WMSServer(Server):
         
         for layer_name in request.params.query_layers:
             layer = self.layers[layer_name]
-            print layer, layer.md, layer.queryable
             if not layer.queryable:
                 raise RequestError('layer %s is not queryable' % layer_name, request=request)
             for layer_name, info_layers in layer.info_layers_for_query(query):
                 actual_layers[layer_name] = info_layers
         
         authorized_info_layers = self.authorized_info_layers(actual_layers.keys(),
-                                                             request.http.environ)
+            request.http.environ, query_extent=(query.srs, query.bbox))
         
         if authorized_info_layers is not PERMIT_ALL_LAYERS:
             requested_layer_names = set(request.params.layers)
@@ -219,7 +216,16 @@ class WMSServer(Server):
 
         return Response(resp, mimetype=mimetype)
     
-    def check_request(self, request):
+    def check_map_request(self, request):
+        self.validate_layers(request)
+        request.validate_format(self.image_formats)
+        request.validate_srs(self.srs)
+    
+    def check_featureinfo_request(self, request):
+        self.validate_layers(request)
+        request.validate_srs(self.srs)
+        
+    def validate_layers(self, request):
         query_layers = request.params.query_layers if hasattr(request, 'query_layers') else []
         for layer in chain(request.params.layers, query_layers):
             if layer not in self.layers:
@@ -246,7 +252,8 @@ class WMSServer(Server):
             mimetype = request.params.format_mime_type
         else:
             mimetype = 'image/png'
-        return Response(result.as_buffer(format=request.params.format).read(), mimetype=mimetype)
+        img_opts = self.image_formats[request.params.format_mime_type]
+        return Response(result.as_buffer(img_opts), mimetype=mimetype)
     
     def _service_md(self, map_request):
         md = dict(self.md)
@@ -254,15 +261,18 @@ class WMSServer(Server):
         md['has_legend'] = self.root_layer.has_legend
         return md
     
-    def authorized_map_layers(self, layers, env):
-        return self._authorize_layers('map', layers, env)
+    def authorized_map_layers(self, layers, env, query_extent):
+        return self._authorize_layers('map', layers, env, query_extent)
     
-    def authorized_info_layers(self, layers, env):
-        return self._authorize_layers('featureinfo', layers, env)
+    def authorized_info_layers(self, layers, env, query_extent):
+        return self._authorize_layers('featureinfo', layers, env, query_extent)
     
-    def _authorize_layers(self, feature, layers, env):
+    def _authorize_layers(self, feature, layers, env, query_extent):
         if 'mapproxy.authorize' in env:
-            result = env['mapproxy.authorize']('wms.' + feature, layers[:])
+            result = env['mapproxy.authorize']('wms.' + feature, layers[:],
+                environ=env, query_extent=query_extent)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
             if result['authorized'] == 'full':
                 return PERMIT_ALL_LAYERS
             layers = set()
@@ -276,7 +286,9 @@ class WMSServer(Server):
     
     def authorized_capability_layers(self, env):
         if 'mapproxy.authorize' in env:
-            result = env['mapproxy.authorize']('wms.capabilities', self.layers.keys())
+            result = env['mapproxy.authorize']('wms.capabilities', self.layers.keys(), environ=env)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
             if result['authorized'] == 'full':
                 return self.root_layer
             if result['authorized'] == 'partial':
@@ -392,7 +404,8 @@ class LayerRenderer(object):
             raise RequestError('Could not get retrieve any sources:\n'+errors, request=self.request)
     
         if errors:
-            layer_merger.add(message_image('\n'.join(errors), self.query.size, transparent=True))
+            layer_merger.add(message_image('\n'.join(errors), self.query.size,
+                image_opts=ImageOptions(transparent=True)))
     
     def _render_layer(self, layer):
         try:
