@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 The WSGI application.
@@ -23,12 +22,13 @@ import os
 import sys
 import site
 
-from paste.registry import RegistryManager
-
 from mapproxy.request import Request
 from mapproxy.response import Response
-import mapproxy.config
-from mapproxy.config.loader import load_configuration
+from mapproxy.util import local_base_config
+from mapproxy.config.loader import load_configuration, ConfigurationError
+
+import logging
+log = logging.getLogger('mapproxy.config')
 
 class Modjyhandler(object):
     def __init__(self):
@@ -106,18 +106,33 @@ def init_null_logging():
             pass
     logging.getLogger().addHandler(NullHandler())
 
-def make_wsgi_app(services_conf=None):
+def make_wsgi_app(services_conf=None, debug=False):
     """
     Create a ProxyApp with the given services conf. Also initializes logging.
     
     :param services_conf: the file name of the services.yaml configuration,
                           if ``None`` the default is loaded.
     """
-    conf = load_configuration(mapproxy_conf=services_conf)
-    services = conf.configured_services()
-
+    try:
+        conf = load_configuration(mapproxy_conf=services_conf)
+        services = conf.configured_services()
+    except ConfigurationError, e:
+        log.fatal(e)
+        raise
+    
     app = MapProxyApp(services, conf.base_config)
-    return RegistryManager(app)
+    if debug:
+        conf.base_config.debug_mode = True
+        try:
+            from werkzeug.debug import DebuggedApplication
+            app = DebuggedApplication(app, evalex=True)
+        except ImportError:
+            try:
+                from paste.evalexception.middleware import EvalException
+                app = EvalException(app)
+            except ImportError:
+                print 'Error: Install Werkzeug or Paste for browser-based debugging.'
+    return app
 
 class MapProxyApp(object):
     """
@@ -127,7 +142,7 @@ class MapProxyApp(object):
     def __init__(self, services, base_config):
         self.handlers = {}
         self.base_config = base_config
-        for service in services.itervalues():
+        for service in services:
             for name in service.names:
                 self.handlers[name] = service
     
@@ -135,28 +150,26 @@ class MapProxyApp(object):
         resp = None
         req = Request(environ)
         
-        registry = environ['paste.registry']
-        registry.register(mapproxy.config.config._config, self.base_config)
-        
-        match = self.handler_path_re.match(req.path)
-        if match:
-            handler_name = match.group(1)
-            if handler_name in self.handlers:
-                try:
-                    resp = self.handlers[handler_name].handle(req)
-                except Exception:
-                    if self.base_config.debug_mode:
-                        raise
-                    else:
-                        import traceback
-                        traceback.print_exc(file=environ['wsgi.errors'])
-                        resp = Response('internal error', status=500)
-        if resp is None:
-            if req.path in ('', '/'):
-                resp = self.welcome_response(req.script_url)
-            else:
-                resp = Response('not found', mimetype='text/plain', status=404)
-        return resp(environ, start_response)
+        with local_base_config(self.base_config):
+            match = self.handler_path_re.match(req.path)
+            if match:
+                handler_name = match.group(1)
+                if handler_name in self.handlers:
+                    try:
+                        resp = self.handlers[handler_name].handle(req)
+                    except Exception:
+                        if self.base_config.debug_mode:
+                            raise
+                        else:
+                            import traceback
+                            traceback.print_exc(file=environ['wsgi.errors'])
+                            resp = Response('internal error', status=500)
+            if resp is None:
+                if req.path in ('', '/'):
+                    resp = self.welcome_response(req.script_url)
+                else:
+                    resp = Response('not found', mimetype='text/plain', status=404)
+            return resp(environ, start_response)
 
     def welcome_response(self, script_url):
         import mapproxy.version
